#define packages to install
rpackages <- c('tidyverse','sampling','partykit',"tidyr",'magrittr','precrec','ebmc',
               'caret','DMwR','MatchIt','pROC','synthpop','zeallot')
#install all packages that are not already installed
install.packages(setdiff(rpackages, rownames(installed.packages())))
lapply(rpackages, require, character.only = TRUE)
###############################################################
library(tidyverse)
library(synthpop)
library(sampling)
library(partykit)
library(tableone)
library(tidyr)
library(ggpubr)
library(magrittr)
library(sjPlot)
library(precrec)
library(ebmc) # ensemble based learning
library(caret)
library(DMwR)
library(MatchIt)
library(pROC)
library(zeallot) 
#library(sdgd)
#library(sdgs) # Not publicly avaiable, function will be replaced with the 
# synthpop r package

## See rcode for the simuated data for the definitions of the functions used here

ros <- function(data) {
  if (mean(data$y) > 0.5) { # check the majority class
    maxClass <- 1
  } else {
    maxClass <- 0
  }
  
  top <- sum(data$y == maxClass) # size of the majority class (to which size the Minority groupneeds to be 'inflated')
  min <- data[data$y != maxClass,] # select Minority groupdata
  ind <- seq(from = 1, to = nrow(min), by = 1) # create vector with row indices Minority groupto sample from
  x <- sample(ind, size = top - nrow(min), replace = TRUE) # sample from row indices with replacement 
  #with the size being the difference between the minority and majority class
  data <- rbind(data, min[x,]) # combine old data with duplicated of the sampled minority cases
  data # return over sampled data set
}

#### RANDOM UNDERSAMPLING FUNCTION ####
# Random undersampling function that downsamples the majority class 

rus <- function(data) {
  if (mean(data$y) < 0.5) { # determine which class is the Minority group
    minClass <- 1
  } else {
    minClass <- 0
  }
  
  top <- sum(data$y == minClass) # Determine size Minority group
  maj <- data[data$y != minClass,] # Select majority class data
  ind <- seq(from = 1, to = nrow(maj), by = 1) # Create vector with indices for the majority class
  x <- sample(ind, size = nrow(maj) - top, replace = FALSE) # sample indices that need to be removed from the original data set
  data <- rbind(data[data$y == minClass,], maj[-x,]) # bind Minority groupdata with remaining majority class data
  data # return undersampled data
}


## Overall function for each subset
LRstandard<-function(trainsample,testsample){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  colnames(testsample)<-c("Y",paste0("X",1:(p-1)))
  
  # format the columns
  cols<-colnames(datt)
  col_cat<-colnames(datt)[1:2]
  datt %<>% mutate_at(col_cat, factor)
  
  modelout <- model <- glm(Y~., data = datt, family = binomial)
  modelcoef<-summary(model)$coef
  
  # Make predictions
  datt<-testsample
  datt$scores <- model %>% predict(testsample, type = "response")
  
  # Calculate ROC and Precision-Recall curves
  sscurves <- evalmod(scores = datt$scores, labels = datt$Y,)
  roc1 <- ci.auc(datt$Y,datt$scores)
  auc = attr(sscurves$rocs[[1]],"auc")
  measures <- evalmod(mode="basic", scores = datt$scores, labels = datt$Y)
  sp = median(na.omit(measures$sp[[1]]$y))
  sn = median(na.omit(measures$sn[[1]]$y))
  fscore = median(na.omit(measures$fscore[[1]]$y))
  
  # Show a Precision-Recall plot
  
  allscore = data.frame(
    matrix(round(c(auc,sp,sn,fscore),4),nrow = 1),
    row.names = NULL)
  colnames(allscore)<-c("auc","sp","sn","fscore")
  allscore<-apply(allscore,2,as.numeric)
  
  return(list(allscore,modelcoef,sscurves,cols_real,modelout,roc1))
}

# rus
RUS_bias<-function(traindata){
  cols<-colnames(traindata)
  trt_level<-levels(traindata[,2])
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("y",paste0("x",1:(p-1)))[c(2,1,3:p)] #sort for rus
  levels(data$y)<-c("0","1")
  data$y<-ifelse(data$y%in%"0",0,1)
  rdata<-rus(data)            # apply rus on the treatment group
  rdata$y<-as.factor(rdata$y) # relevel the treatment to original
  levels(rdata$y)<-trt_level
  colnames(rdata)<-cols
  return(rdata)
}


### ROS
ROS_bias<-function(traindata){
  cols<-colnames(traindata)
  trt_level<-levels(traindata[,2])
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("y",paste0("x",1:(p-1)))[c(2,1,3:p)] #sort for rus
  levels(data$y)<-c("0","1")
  data$y<-ifelse(data$y%in%"0",0,1)
  rdata<-ros(data)            # apply ros on the treatment group
  rdata$y<-as.factor(rdata$y) # relevel the treatment to original
  levels(rdata$y)<-trt_level
  colnames(rdata)<-cols
  return(rdata)
}


### SMOTE
SMOTE_bias<-function(traindata,perc=100){
  cols<-colnames(traindata)
  trt_level<-levels(traindata[,2])
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("y",paste0("x",1:(p-1)))[c(2,1,3:p)] #sort for rus
  levels(data$y)<-c("0","1")
  data$y<-as.factor(ifelse(data$y%in%"0",0,1))
  rdata<-SMOTE(y ~ ., data, perc.over = perc)     # apply smote
  rdata$y<-as.factor(rdata$y)                     # relevel 
  levels(rdata$y)<-trt_level
  colnames(rdata)<-cols
  return(rdata)
}


### Propensity score matching
LR_PSbias<-function(trainsample,testsample){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  # format the columns
  colnames(testsample)<-cols<-colnames(datt)
  col_cat<-colnames(datt)[1:2]
  datt %<>% mutate_at(col_cat, factor)
  
  # repeat for the testset
  testsample %<>% mutate_at(col_cat, factor)
  
  Trt<-"X1"
  Variables<-cols[3:p]
  f<-as.formula(
    paste(Trt,paste(Variables,collapse = "+"),sep = "~")
  )
  m.out <- matchit(f, data = datt,
                   method = "nearest", distance = "glm")
  datt <- match.data(m.out)
  # repeat for testset
  t.out <- matchit(f, data = testsample,
                   method = "nearest", distance = "glm")
  testsample <- match.data(t.out)
  ###
  Outcome<-"Y"
  Variables<-cols[2:p]
  fmodel<-as.formula(
    paste(Outcome,paste(Variables,collapse = "+"),sep = "~")
  )
  modelout <- model <- glm(fmodel, data = datt, family = binomial,
                           weights = weights)
  modelcoef<-summary(model)$coef
  
  # Make predictions
  datt<-testsample
  datt$scores <- model %>% predict(testsample, type = "response")
  
  # Calculate ROC and Precision-Recall curves
  sscurves <- evalmod(scores = datt$scores, labels = datt$Y)
  roc1 <- ci.auc(datt$Y,datt$scores)
  
  
  auc = attr(sscurves$rocs[[1]],"auc")
  measures <- evalmod(mode="basic", scores = datt$scores, labels = datt$Y)
  sp = median(na.omit(measures$sp[[1]]$y))
  sn = median(na.omit(measures$sn[[1]]$y))
  fscore = median(na.omit(measures$fscore[[1]]$y))
  
  # Show a Precision-Recall plot
  #autoplot(sscurves, "PRC")
  
  allscore = data.frame(
    matrix(round(c(auc,sp,sn,fscore),4),nrow = 1),
    row.names = NULL)
  colnames(allscore)<-c("auc","sp","sn","fscore")
  allscore<-apply(allscore,2,as.numeric)
  
  return(list(allscore,modelcoef,sscurves,cols_real,modelout,roc1))
}

### Propensity score matching
LR_PSbias2<-function(trainsample,testsample,nclass){
  cols_real<- colnames(trainsample)
  datt<-trainsample
  n = dim(datt)[1]; p = dim(datt)[2]
  
  colnames(datt)<-c("Y",paste0("X",1:(p-1)))
  # format the columns
  colnames(testsample)<-cols<-colnames(datt)
  col_cat<-colnames(datt)[1:2]
  datt %<>% mutate_at(col_cat, factor)
  
  # repeat for the testset
  testsample %<>% mutate_at(col_cat, factor)
  
  
  Trt<-"X1"
  Variables<-cols[3:p]
  f<-as.formula(
    paste(Trt,paste(Variables,collapse = "+"),sep = "~")
  )
  m.out <- matchit(f, data = datt,
                   method = "nearest", distance = "glm")
  datt <- match.data(m.out)
  # repeat for testset
  t.out <- matchit(f, data = testsample,
                   method = "nearest", distance = "glm")
  testsample <- match.data(t.out)
  ###
  Outcome<-"Y"
  Variables<-cols[2:p]
  fmodel<-as.formula(
    paste(Outcome,paste(Variables,collapse = "+"),sep = "~")
  )
  modelout <- model <- glm(fmodel, data = datt, family = binomial,
                           weights = weights)
  modelcoef<-summary(model)$coef
  
  # Make predictions
  testsample<-testsample[testsample$X1==levels(testsample$X1)[nclass],]
  datt<-testsample
  datt$scores <- model %>% predict(testsample, type = "response")
  
  # Calculate ROC and Precision-Recall curves
  sscurves <- evalmod(scores = datt$scores, labels = datt$Y)
  roc1 <- ci.auc(datt$Y,datt$scores)
  
  
  auc = attr(sscurves$rocs[[1]],"auc")
  measures <- evalmod(mode="basic", scores = datt$scores, labels = datt$Y)
  sp = median(na.omit(measures$sp[[1]]$y))
  sn = median(na.omit(measures$sn[[1]]$y))
  fscore = median(na.omit(measures$fscore[[1]]$y))
  
  # Show a Precision-Recall plot
  
  allscore = data.frame(
    matrix(round(c(auc,sp,sn,fscore),4),nrow = 1),
    row.names = NULL)
  colnames(allscore)<-c("auc","sp","sn","fscore")
  allscore<-apply(allscore,2,as.numeric)
  
  return(list(allscore,modelcoef,sscurves,cols_real,modelout,roc1))
}

# Functions for the 95% Confidence Interval Overlaps based on XXXX
interval_overlap<-function(L1,U1,L2,U2){
  if (L1==L2 & U1==U2) {
    I = 1
  }
  if (U1<L2 | U2<L1) {
    I = 0
  }
  if (L1<=L2 & L2<U1 & U1<=U2) {
    Li = L2
    Ui = U1
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  if (L1<=L2 & L2<U1 & U2<U1) {
    Li = L2
    Ui = U2
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  
  if (L2<=L1 & L1<U2 & U2<=U1) {
    Li = L1
    Ui = U2
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  if (L2<=L1 & L1<U2 & U1<U2) {
    Li = L1
    Ui = U1
    I = (Ui-Li)/(2*(U1-L1)) + (Ui-Li)/(2*(U2-L2))
  }
  return(I)
}

vec_int_overlap<-function(vec1,vec2){
  out_vec<-rep(NA,length(vec1))
  for (i in 1:length(out_vec)) {
    out_vec[i]<-interval_overlap(vec1[1],vec2[1],vec1[i],vec2[i])
  }
  return(out_vec)
}


sdgs_synthesis<-function(realdata){ # replace this with the synthesis function in synthpop
  real.dataset<-realdata %>% na.omit
  yy<-sdgs::synthpop$new()
  # set
  c(yy$verbose, yy$tune , yy$calibrate, yy$parallel) %<-% c(FALSE, FALSE, FALSE, TRUE)
  yy$dataset<-real.dataset
  syn.dataset<-yy$synthesize(nrow(real.dataset))
  yy$finalize()
  return(syn.dataset$syn_data)
}


## Methods evaluated for the Breast Cancer data from UCI machine learning repository

#1. Original (Gold standard)
#2. Biased data
#3. Random Undersampling (RUS)
#4. Random Oversampling (ROS)
#5. Synthetic Minority Oversampling Technique (SMOTE)
#6. PS Matching
#7. Oversampling from the Minority group
#8. Stratified analysis


# Fix the parameter
prop_val = 0.3

BCdata<-readRDS(file = "UCI_breastcancer_data_clean.rds")
col.cols<-colnames(BCdata)[c(10,1,7,2,5,6,9)]
subdata<-BCdata[,col.cols]
levels(subdata$Class)<-c("no_recur","recur")
col_cat<-col.cols[1:2]
subdata %<>% mutate_at(col_cat, factor)
myVars<-col.cols[-2]

# Create the tableone for the population
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2], data =subdata)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))

# the test and train sets
test.sel<-sample(1:dim(subdata)[1], size = round(dim(subdata)[1]*.7),replace = F)
traindata<-na.omit(subdata[-test.sel,])
testdata<-na.omit(subdata[test.sel,])

# Tableone for the trainset
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =traindata)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


## Model for the full cohort data
goldstandard<-LRstandard(trainsample = traindata,testsample = testdata)
(prevscore=goldstandard[[1]])
goldstandard[[6]]
(dd=goldstandard[[2]])
goldstandard[[3]]
autoplot(goldstandard[[3]])


# summarize the results
beta_true = goldstandard[[2]][2,1]
SE_true = goldstandard[[2]][2,2]
LL_true = exp(beta_true-1.96*SE_true)
UL_true = exp(beta_true+1.96*SE_true)

beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

# the AUC, OR< SE , and Confidence Interval Overlap
cat("\n  AUC | OR | SE | Overlap :\n")
(orig_gold = c(prevscore[1],exp(estbar),SE,Overlap[2]))


# Minority group results
goldstandard2<-LRstandard(trainsample = traindata,
                          testsample = testdata[testdata$age==levels(testdata$age)[1],])
goldstandard2[[1]]
goldstandard2[[6]]
goldstandard2[[2]]
goldstandard2[[3]]
autoplot(goldstandard2[[3]])


### Inducing the different types of biases

# Type I bias: Marginal bias
Induce_bias<-function(traindata,prop_out){
  cols<-colnames(traindata)
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("Y",paste0("X",1:(p-1)))
  samps<-floor(table(data$X1)[1]*prop_out)
  data = data[sample(-which(data$X1%in%levels(data$X1)[1]),samps),]
  colnames(data)<-cols
  return(data)
}

# Define the function for generating the synthetic sampole
augment_synth_mar<-function(biased_data,props){
  data_bias2<-biased_data 
  #data_bias2<-data.frame(apply(data_bias2,2,as.numeric))
  
  # remove missing
  real.dataset<-data_bias2 %>% na.omit
  
  # Generate the synthetic data 
  synth<-sdgs_synthesis(real.dataset)
  countminor<-length(which(biased_data[,2]%in%levels(biased_data[,2])[1]))
  prop_remain<-1-props
  total_size<-floor(countminor/prop_remain)
  synth.sel<-synth[sample(x = which(synth[,2]%in%levels(biased_data[,2])[1]),
                          size = (total_size-countminor), replace = T),]
  augment_syn<-rbind(real.dataset,synth.sel)
  return(augment_syn)
}


data_bias<-Induce_bias(traindata = traindata, prop_out = prop_val)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_bias)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# For the full cohort
LRbias<-LRstandard(trainsample = data_bias,
                   testsample = testdata)
(prevscore=LRbias[[1]])
LRbias[[6]]
(dd = LRbias[[2]])
LRbias[[3]]
autoplot(LRbias[[3]])
beta_new = dd[2,1]

SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(mar_bias = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority category
LRbias2<-LRstandard(trainsample = data_bias,
                    testsample = testdata[testdata$age==levels(testdata$age)[1],])

LRbias2[[1]]
LRbias2[[6]]
LRbias2[[2]]
LRbias2[[3]]
autoplot(LRbias2[[3]])

### RUS
data_rus<-RUS_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_rus)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))

# Full Cohort
LRrus<-LRstandard(trainsample = data_rus,testsample = testdata)
(prevscore=LRrus[[1]])
LRrus[[6]]
(dd = LRrus[[2]])
LRrus[[3]]
autoplot(LRrus[[3]])

beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(mar_rus = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group
LRrus2<-LRstandard(trainsample = data_rus,
                   testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRrus2[[1]]
LRrus2[[6]]
LRrus2[[2]]
LRrus2[[3]]
autoplot(LRrus2[[3]])

### ROS
data_ros<-ROS_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_ros)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))

# Full cohort

LRros<-LRstandard(trainsample = data_ros,
                  testsample = testdata)
(prevscore=LRros[[1]])
LRros[[6]]
(dd =LRros[[2]])
LRros[[3]]
autoplot(LRros[[3]])

beta_new = dd[2,1]


SE = dd[2,2]

cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(mar_ros = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority category
LRros2<-LRstandard(trainsample = data_ros,
                   testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRros2[[1]]
LRros2[[6]]
LRros2[[2]]
LRros2[[3]]
autoplot(LRros2[[3]])


### SMOTE



data_smote<-SMOTE_bias(data_bias,perc=1000)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_smote)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))



# Full Cohort


LRsmote<-LRstandard(trainsample = data_smote,
                    testsample = testdata)

(prevscore=LRsmote[[1]])
LRsmote[[6]]
(dd = LRsmote[[2]])
LRsmote[[3]]

autoplot(LRsmote[[3]])







beta_new = dd[2,1]



SE = dd[2,2]


cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]




cat("\n  AUC | OR | SE | Overlap :\n")
(mar_smote = c(prevscore[1],exp(estbar),SE,Overlap[2]))



# Minority Category
LRsmote2<-LRstandard(trainsample = data_smote,
                     testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRsmote2[[1]]
LRsmote2[[6]]
LRsmote2[[2]]
LRsmote2[[3]]

autoplot(LRsmote2[[3]])

### Propensity score matching

# Full Cohort
LRps<-LR_PSbias(trainsample = data_bias,
                testsample = testdata)
(prevscore=LRps[[1]])
LRps[[6]]
(dd = LRps[[2]])
LRps[[3]]
autoplot(LRps[[3]])

beta_new = dd[2,1]


SE = dd[2,2]

cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(mar_ps = c(prevscore[1],exp(estbar),SE,Overlap[2]))


#Minority group
LRps2<-LR_PSbias2(trainsample = data_bias,
                  testsample = testdata,nclass = 1)
LRps2[[1]]
LRps2[[6]]
LRps2[[2]]
LRps2[[3]]
autoplot(LRps2[[3]])

### Synthetic Minor Augmentation
augment_syn<-augment_synth_mar(biased_data = data_bias, props=prop_val)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2], data =augment_syn)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))



#Full cohort

#augment_syn$Y<-as.integer(augment_syn$Y)
syn_LRbias<-LRstandard(trainsample = augment_syn,
                       testsample = testdata)
(prevscore=syn_LRbias[[1]])
syn_LRbias[[6]]
(dd = syn_LRbias[[2]])
syn_LRbias[[3]]
autoplot(syn_LRbias[[3]])

beta_new = dd[2,1]



SE = dd[2,2]

cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(mar_syn = c(prevscore[1],exp(estbar),SE,Overlap[2]))

#Minority group
syn_LRbias2<-LRstandard(trainsample = augment_syn,
                        testsample = testdata[testdata$age==levels(testdata$age)[1],])
syn_LRbias2[[1]]
syn_LRbias2[[6]]
syn_LRbias2[[2]]
syn_LRbias2[[3]]
autoplot(syn_LRbias2[[3]])

## Conditional bias I
### Original data
#Treatment Age is weakly/moderately associated with the right/left breast region
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =traindata)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


### Biased data
Induce_bias_1<-function(traindata,prop_out){
  cols<-colnames(traindata)
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("Y",paste0("X",1:(p-1)))
  selval<-which(data$X1%in%levels(data$X1)[1]&
                  data$X2%in%levels(data$X2)[1])
  samps<-floor(length(selval)[1]*prop_out)
  data = data[sample(-selval,samps),]
  colnames(data)<-cols
  return(data)
}

augment_synth_cond1<-function(biased_data,props){
  data_bias2<-biased_data
  #data_bias2<-data.frame(apply(data_bias2,2,as.numeric))
  # remove missing
  real.dataset<-data_bias2 %>% na.omit
  
  # Generate the synthetic data with CART
  synth<-sdgs_synthesis(real.dataset)
  countminor<-length(which(biased_data[,2]%in%levels(biased_data[,2])[1]&
                             biased_data[,3]%in%levels(biased_data[,3])[1]))
  prop_remain<-1-props
  total_size<-floor(countminor/prop_remain)
  synth.sel<-synth[sample(x = which(synth[,2]%in%levels(biased_data[,2])[1]&synth[,3]%in%levels(biased_data[,3])[1]),
                          size = (total_size-countminor), replace = T),]
  return(augment_syn)
}

data_bias<-Induce_bias_1(traindata = traindata, prop_out = prop_val)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_bias)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# Full Cohort
LRbias<-LRstandard(trainsample = data_bias,
                   testsample = testdata)
(prevscore=LRbias[[1]])
(dd = LRbias[[2]])
LRbias[[6]]
LRbias[[3]]
autoplot(LRbias[[3]])

beta_new = dd[2,1]


SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(cond1_bias = c(prevscore[1],exp(estbar),SE,Overlap[2]))


#Minority group
LRbias2<-LRstandard(trainsample = data_bias,
                    testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRbias2[[1]]
LRbias2[[6]]
LRbias2[[2]]
LRbias2[[3]]
autoplot(LRbias2[[3]])

### RUS
data_rus<-RUS_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_rus)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# Full Cohort
LRrus<-LRstandard(trainsample = data_rus,
                  testsample = testdata)
(prevscore=LRrus[[1]])
LRrus[[6]]
(dd = LRrus[[2]])
LRrus[[3]]
autoplot(LRrus[[3]])

beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(cond1_rus= c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group

LRrus2<-LRstandard(trainsample = data_rus,
                   testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRrus2[[1]]
LRrus2[[6]]
LRrus2[[2]]
LRrus2[[3]]
autoplot(LRrus2[[3]])

# Majority class 

LRrus2<-LRstandard(trainsample = data_rus,
                   testsample = testdata[testdata$age==levels(testdata$age)[2],])
LRrus2[[1]]
LRrus2[[6]]
LRrus2[[2]]
LRrus2[[3]]
autoplot(LRrus2[[3]])

### ROS
data_ros<-ROS_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_ros)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# Full Cohort
LRros<-LRstandard(trainsample = data_ros,
                  testsample = testdata)
(prevscore=LRros[[1]])
LRros[[6]]
(dd = LRros[[2]])
LRros[[3]]
autoplot(LRros[[3]])

beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(cond1_ros = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group
LRros2<-LRstandard(trainsample = data_ros,
                   testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRros2[[1]]
LRros2[[6]]
LRros2[[2]]
LRros2[[3]]
autoplot(LRros2[[3]])


### SMOTE

data_smote<-SMOTE_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_smote)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# Full Cohort
LRsmote<-LRstandard(trainsample = data_smote,
                    testsample = testdata)
(prevscore=LRsmote[[1]])
LRsmote[[6]]
(dd = LRsmote[[2]])
LRsmote[[3]]
autoplot(LRsmote[[3]])


beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(cond1_smote= c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group
LRsmote2<-LRstandard(trainsample = data_smote,
                     testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRsmote2[[1]]
LRsmote2[[6]]
LRsmote2[[2]]
LRsmote2[[3]]
autoplot(LRsmote2[[3]])


### Propensity score matching

# Full Cohort
LRps<-LR_PSbias(trainsample = data_bias,
                testsample = testdata)
(prevscore=LRps[[1]])
LRps[[6]]
(dd = LRps[[2]])
LRps[[3]]
autoplot(LRps[[3]])


beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(cond1_ps  = c(prevscore[1],exp(estbar),SE,Overlap[2]))


# Minority group

LRps2<-LR_PSbias2(trainsample = data_bias,
                  testsample = testdata,nclass = 1)
LRps2[[1]]
LRps2[[6]]
LRps2[[2]]
LRps2[[3]]
autoplot(LRps2[[3]])


### Synthetic Minor Augmentation
augment_syn<-augment_synth_cond1(biased_data = data_bias, props = prop_val )
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2], data =augment_syn)
print(tab1, showAllLevels = TRUE, formatOptions = list(big.mark = ","))


# Full Cohort
syn_LRbias<-LRstandard(trainsample = augment_syn,
                       testsample = testdata)
(prevscore=syn_LRbias[[1]])
syn_LRbias[[6]]
(dd = syn_LRbias[[2]])
syn_LRbias[[3]]
autoplot(syn_LRbias[[3]])

beta_new = dd[2,1]


SE = dd[2,2]

cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(cond1_syn = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group
syn_LRbias2<-LRstandard(trainsample = augment_syn,
                        testsample = testdata[testdata$age==levels(testdata$age)[1],])
syn_LRbias2[[1]]
syn_LRbias2[[6]]
syn_LRbias2[[2]]
syn_LRbias2[[3]]
autoplot(syn_LRbias2[[3]])




## Conditional bias II
# Treatment Age is strongly associated with bmi

Induce_bias_2<-function(traindata,prop_out){
  cols<-colnames(traindata)
  data<-traindata
  n = dim(data)[1]; p = dim(data)[2]
  colnames(data)<-c("Y",paste0("X",1:(p-1)))
  samps<-floor(length(which(data$X1%in%levels(data$X1)[1]&
                              data$X3%in%levels(data$X3)[1]))[1]*prop_out)
  data = data[sample(-which(data$X1%in%levels(data$X1)[1]&data$X3%in%levels(data$X3)[1]),samps),]
  colnames(data)<-cols
  return(data)
}

augment_synth_cond2<-function(biased_data,props){
  data_bias2<-biased_data
  #data_bias2<-data.frame(apply(data_bias2,2,as.numeric))
  # remove missing
  real.dataset<-data_bias2 %>% na.omit
  
  # Generate the synthetic data with CART
  synth<-sdgs_synthesis(real.dataset)
  countminor<-length(which(biased_data[,2]%in%levels(biased_data[,2])[1]&
                             biased_data[,4]%in%levels(biased_data[,4])[1]))
  prop_remain<-1-props
  total_size<-floor(countminor/prop_remain)
  
  synth.sel<-synth[sample(x = which(biased_data[,2]%in%levels(biased_data[,2])[1]&
                                      biased_data[,4]%in%levels(biased_data[,4])[1]),
                          size = (total_size-countminor), replace = T),]
  augment_syn<-rbind(real.dataset,synth.sel)
  
  return(augment_syn)
}


# Biased data 
data_bias<-Induce_bias_2(traindata = traindata, prop_out = prop_val)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_bias)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))

# Full Cohort
LRbias<-LRstandard(trainsample = data_bias,
                   testsample = testdata)
(prevscore=LRbias[[1]])
LRbias[[6]]
(dd = LRbias[[2]])
LRbias[[3]]

autoplot(LRbias[[3]])



beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(cond2_bias = c(prevscore[1],exp(estbar),SE,Overlap[2]))


# Minority group
LRbias2<-LRstandard(trainsample = data_bias,
                    testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRbias2[[1]]
LRbias2[[3]]
LRbias2[[2]]
LRbias2[[3]]
autoplot(LRbias2[[3]])


### RUS

data_rus<-RUS_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_rus)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# Full Cohort
LRrus<-LRstandard(trainsample = data_rus,
                  testsample = testdata)
(prevscore=LRrus[[1]])
LRrus[[6]]
(dd = LRrus[[2]])
LRrus[[3]]
autoplot(LRrus[[3]])

beta_new = dd[2,1]


SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(cond2_rus = c(prevscore[1],exp(estbar),SE,Overlap[2]))


# Minority group
LRrus2<-LRstandard(trainsample = data_rus,
                   testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRrus2[[1]]
LRrus2[[6]]
LRrus2[[2]]
LRrus2[[3]]
autoplot(LRrus2[[3]])


### ROS

data_ros<-ROS_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_ros)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# Full Cohort
LRros<-LRstandard(trainsample = data_ros,
                  testsample = testdata)
(prevscore=LRros[[1]])
LRros[[6]]
(dd = LRros[[2]])
LRros[[3]]

autoplot(LRros[[3]])


beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]
cat("\n  AUC | OR | SE | Overlap :\n")
(cond2_ros = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group
LRros2<-LRstandard(trainsample = data_ros,
                   testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRros2[[1]]
LRros2[[6]]
LRros2[[2]]
LRros2[[3]]
autoplot(LRros2[[3]])


### SMOTE

data_smote<-SMOTE_bias(data_bias)
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2] ,
                       data =data_smote)
print(tab1, showAllLevels = TRUE,formatOptions = list(big.mark = ","))


# Full Cohort
LRsmote<-LRstandard(trainsample = data_smote,
                    testsample = testdata )
(prevscore=LRsmote[[1]])
LRsmote[[6]]
(dd = LRsmote[[2]])
LRsmote[[3]]
autoplot(LRsmote[[3]])

beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(cond2_smote = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group
LRsmote2<-LRstandard(trainsample = data_smote,
                     testsample = testdata[testdata$age==levels(testdata$age)[1],])
LRsmote2[[1]]
LRsmote2[[6]]
LRsmote2[[2]]
LRsmote2[[3]]
autoplot(LRsmote2[[3]])

# Propensity score matching

# Full Cohort
LRps<-LR_PSbias(trainsample = data_bias,
                testsample = testdata)
(prevscore=LRps[[1]])
LRps[[6]]
(dd = LRps[[2]])
LRps[[3]]
autoplot(LRps[[3]])

beta_new = dd[2,1]


SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(cond2_ps = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group
LRps2<-LR_PSbias2(trainsample = data_bias,
                  testsample = testdata,nclass = 1)
LRps2[[1]]
LRps2[[6]]
LRps2[[2]]
LRps2[[3]]
autoplot(LRps2[[3]])

### Synthetic Minor Augmentation
augment_syn<-augment_synth_cond2(biased_data = data_bias, props = prop_val )
tab1 <- CreateTableOne(vars = myVars, strata = col.cols[2], data =augment_syn)
print(tab1, showAllLevels = TRUE, formatOptions = list(big.mark = ","))


# Full Cohort

syn_LRbias<-LRstandard(trainsample = augment_syn,
                       testsample = testdata)
(prevscore=syn_LRbias[[1]])
syn_LRbias[[6]]
(dd = syn_LRbias[[2]])
syn_LRbias[[3]]
autoplot(syn_LRbias[[3]])

beta_new = dd[2,1]



SE = dd[2,2]
cat("\n 95% CI and Overlap: \n")
estbar = beta_new
LL = c(LL_true,exp(estbar-1.96*SE))
UL = c(UL_true,exp(estbar+1.96*SE))
Overlap =vec_int_overlap(LL,UL)
t(rbind(LL,exp(estbar),UL,Overlap))[2,]

cat("\n  AUC | OR | SE | Overlap :\n")
(cond2_syn = c(prevscore[1],exp(estbar),SE,Overlap[2]))

# Minority group analysis
syn_LRbias2<-LRstandard(trainsample = augment_syn,
                        testsample = testdata[testdata$age==levels(testdata$age)[1],])
syn_LRbias2[[1]]
syn_LRbias2[[6]]
syn_LRbias2[[2]]
syn_LRbias2[[3]]
autoplot(syn_LRbias2[[3]])




